%global with_debug 1

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%global vendor_stdlib golang.org/x/net/http golang.org/x/net/http2/hpack golang.org/x/net/idna

%if 0%{?rhel} >= 8
%define gobuild(o:) go build -mod vendor -buildmode pie -tags=rpm_crashtraceback -ldflags "${LDFLAGS:-} -compressdwarf=false -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -linkmode external -extldflags '-Wl,-z,relro  -Wl,-z,now -specs=/usr/lib/rpm/redhat/redhat-hardened-ld'" -a -v -x %{?**};
%else
%define gobuild(o:) go build -mod vendor -buildmode pie -tags=rpm_crashtraceback -ldflags "${LDFLAGS:-} -compressdwarf=false -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -linkmode external -extldflags '-Wl,-z,relro,-z,now'" -a -v -x %{?**};
%endif

%global provider        github
%global provider_tld    com
%global project         coreos
%global repo            etcd
# https://github.com/coreos/etcd
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}

%global system_name     etcd
%global man_version     3.3.23

Name:		etcd
Version:	3.3.23
Release:	16%{?dist}
Summary:	A highly-available key value store for shared configuration
License:	ASL 2.0
URL:		https://%{provider_prefix}
#Source0:	https://%{provider_prefix}/archive/v%{version}.tar.gz
Source0:        %{system_name}-%{version}.tar.gz
Source1:	%{system_name}.service
Source2:	%{system_name}.conf
Source3:        man-%{man_version}.tar.gz
Source4:	fail_on_h2_bundle_changes.sh

Patch0001: 0001-DOWNSTREAM-ONLY-Add-keep-cluster-id-and-node-id-to-e.patch
Patch0002: 0002-Rebase-vendored-golang.org-x-net-http2-to-v0.24.0.patch
Patch0003: 0003-CVE-2022-21698-promhttp-Check-validity-of-method-and.patch
Patch0004: 0004-CVE-2022-3064-Update-vendored-yaml.v2-to-2.2.4.patch
Patch0005: 0005-CVE-2021-28235.patch

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:x86_64 aarch64 ppc64le s390x}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

Obsoletes: etcd3 < 3.0.15
Provides: etcd3 = %{version}-%{release}

BuildRequires: libpcap-devel

BuildRequires:	systemd

Requires(pre):	shadow-utils
Requires(post): systemd
Requires(preun): systemd
Requires(postun): systemd

%description
A highly-available key value store for shared configuration.

%{?enable_gotoolset7}

%prep
%setup -q -n man-%{man_version} -T -b 3
%setup -q -n %{repo}-%{version}

VENDOR_STDLIB=(%{vendor_stdlib})
echo $VENDOR_STDLIB
# (migarcia) RHEL's golang package vendors a lot of the x/net code etcd needs.
# Let's use it so that we benefit from CVE backports.
for module in "${VENDOR_STDLIB[@]}"; do
  rm -rf vendor/${module}
done

%patch0001 -p1
%patch0002 -p1
%patch0003 -p1
%patch0004 -p1
%patch0005 -p1

# fail_on_h2_bundle_changes.sh: fail if RHEL's bundled x/net/http2 code has any changes,
# as they could be CVE fixes that we need to manually apply to our vendored copy. We pass in
# the sha256sum of the latest-known h2_bundle.go and the exact RPM it belonged to.
# We can update this sha256sum once we're sure we applied any new needed patches to our vendored copy.
chmod +x %{SOURCE4}
%{SOURCE4} 937280864b732293e59d82a5487c27b60d869e0396e1a14d5b59b01960e57c5c golang-src-1.20.12-8.el8_9.noarch

mkdir -p man/man1
cp ../man-%{man_version}/*.1 man/man1/.

%build
export CGO_CFLAGS="%{optflags}"

# Avoid the build process trying to import etcd itself as a bundled dependency
find . -name "*.go" -exec sed -i "s|github.com/coreos/etcd|go.etcd.io/etcd|" "{}" +;

# (migarcia) RHEL's golang package vendors a lot of the x/net code etcd needs.
# Let's use it so that we benefit from CVE backports.
mkdir -p Godeps/_stdlib/src
VENDOR_STDLIB=(%{vendor_stdlib})
for module in "${VENDOR_STDLIB[@]}"; do
  mkdir -p vendor/${module}
  # We need to actually copy the files because the Go compiler really doesn't like
  # symlinking out in import paths.
  cp -a /usr/lib/golang/src/vendor/${module}/. vendor/${module}
done

%gobuild -o bin/%{system_name}
%gobuild -o bin/%{system_name}ctl ./%{system_name}ctl

%install
install -D -p -m 0755 bin/%{system_name} %{buildroot}%{_bindir}/%{system_name}
install -D -p -m 0755 bin/%{system_name}ctl %{buildroot}%{_bindir}/%{system_name}ctl
install -D -p -m 0644 %{SOURCE1} %{buildroot}%{_unitdir}/%{system_name}.service
install -d -m 0755 %{buildroot}%{_sysconfdir}/%{system_name}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{system_name} %{SOURCE2}

# install manpages
install -d %{buildroot}%{_mandir}/man1
install -p -m 644 man/man1/* %{buildroot}%{_mandir}/man1

# And create /var/lib/etcd
install -d -m 0755 %{buildroot}%{_sharedstatedir}/%{system_name}

%pre
getent group %{system_name} >/dev/null || groupadd -r %{system_name}
getent passwd %{system_name} >/dev/null || useradd -r -g %{system_name} -d %{_sharedstatedir}/%{system_name} \
	-s /sbin/nologin -c "etcd user" %{system_name}

%post
%systemd_post %{system_name}.service

%preun
%systemd_preun %{system_name}.service

%postun
%systemd_postun %{system_name}.service

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
%license LICENSE
%doc *.md
%doc glide.lock
%config(noreplace) %{_sysconfdir}/%{system_name}
%{_bindir}/%{system_name}
%{_bindir}/%{system_name}ctl
%dir %attr(-,%{system_name},%{system_name}) %{_sharedstatedir}/%{system_name}
%{_unitdir}/%{system_name}.service
%{_mandir}/man1/*.1*

%changelog
* Wed Apr 24 2024 Miguel Garcia <migarcia@redhat.com> 3.3.23-16
- Rebase vendored golang.org/x/net/http2 to v0.24.0 
- Stop vendoring some golang.org/x/net modules that are provided by RHEL's golang
- Add build-time check for changes in RHEL's bundled x/net/http2 code 

* Fri May 19 2023 Lon Hohberger <lon@redhat.com> - 3.3.23-13
- Fix CVE-2021-28235

* Tue Jan 31 2023 Lon Hohberger <lon@redhat.com> - 3.3.23-12
- CVE-2022-3064 go-yaml: Improve heuristics preventing CPU/memory abuse by parsing malicious or large YAML documents
- Rebuild with new golang

* Tue Aug 02 2022 Miguel Garcia <migarcia@redhat.com> - 3.3.23-10
- Rebuild with new golang

* Fri Jun 03 2022 Miguel Garcia <mgarciac@redhat.com> - 3.3.23-9
- CVE-2022-21698 prometheus/client_golang: promhttp: Check validity of method and code label values (rhbz#2067475)

* Wed Jan 19 2022 Lon Hohberger <lon@redhat.com> - 3.3.23-8
- Honor buildroot-supplied CFLAGS during build

* Tue Jan 11 2022 Miguel Garcia <mgarciac@redhat.com> - 3.3.23-7
- Rebuild with new golang

* Wed Dec 01 2021 Miguel Garcia <mgarciac@redhat.com> - 3.3.23-6
- Rebuild with new golang

* Wed Jun 30 2021 Miguel Garcia <mgarciac@redhat.com> 3.3.23-3
- CVE-2021-31525 net/http: remove recursion in HeaderValuesContainsToken (rhbz#1961022)

* Thu Jun 10 2021 Lon Hohberger <lon@redhat.com> - 3.3.23-2
- Correct hardening on RHEL8

* Thu Dec 03 2020 Miguel Garcia <mgarciac@redhat.com> - 3.3.23-1
- Update to 3.3.23
- [DOWNSTREAM ONLY] Add --keep-cluster-id and --node-id to 'etcdctl backup' (rhbz#1350875)

* Wed Mar 06 2019 Lon Hohberger <lon@redhat.com> - 3.2.21-2
- Rebuild for new architectures

* Mon Jun 04 2018 Jan Chaloupka <jchaloup@redhat.com> - 3.2.21-1
- Update to 3.2.21
  resolves: #1585787

* Tue Apr 17 2018 Jan Chaloupka <jchaloup@redhat.com> - 3.2.18-1
- Update to 3.2.18
  resolves: #1568389

* Tue Feb 06 2018 Jan Chaloupka <jchaloup@redhat.com> - 3.2.15-2
- Rebuild for 7.5.0
  resolves: #1542526

* Mon Jan 29 2018 Jan Chaloupka <jchaloup@redhat.com> - 3.2.15-1
- Update to 3.2.15
  resolves: #1539670
  resolves: #1378706

* Wed Dec 06 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.11-1
- Update to 3.2.11
  related: #1514612

* Tue Nov 21 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.10-1
- Update to 3.2.10
  resolves: #1514612

* Mon Nov 20 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.9-3
- Bump man-pages tarball
  related: #1510480

* Mon Nov 20 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.9-2
- Fix Synopsis of etcdctl3 man pages
  related: #1510480

* Tue Nov 07 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.9-1
- Update to 3.2.9
  resolves: #1510480
- Generate etcd and etcdctl man-pages
  resolves: #1444336

* Fri Sep 29 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.7-2
- Rebuild with correct hardening flags
  resolves: #1420783

* Tue Sep 19 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.7-1
- Update to 3.2.7
  resolves: #1493165

* Tue Aug 08 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.2.5-1
- Update to 3.2.5
  resolves: #1479371

* Mon Jun 12 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.9-2
- Build for secondary architectures as well

* Fri Jun 09 2017 Scott Dodson <sdodson@redhat.com> - 3.1.9-1
- Update to 3.1.9
  resolves: #1458941

* Tue Jun 06 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.8-1
- Update to 3.1.8
  resolves: #1459122

* Tue May 02 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.7-1
- Update to 3.1.7
  resolves: #1447235

* Tue Apr 04 2017 Yaakov Selkowitz <yselkowi@redhat.com> - 3.1.3-2
- Circumvent runtime check of officially supported architectures
  resolves: #1434973

* Tue Mar 21 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.3-1
- Update to 3.1.3
  resolves: #1434364

* Mon Feb 27 2017 Josh Boyer <jwboyer@redhat.com> - 3.1.0-2.1
- Rebuild rebase on all architectures

* Tue Feb 21 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.0-2
- Apply "add --keep-cluster-id and --node-id to 'etcdctl backup'"
  from extras-rhel-7.2 branch
  resolves: #1350875

* Thu Feb 16 2017 Josh Boyer <jwboyer@redhat.com> - 3.1.0-1.1
- Rebuild rebase on all architectures

* Mon Feb 06 2017 Jan Chaloupka <jchaloup@redhat.com> - 3.1.0-1
- Update to 3.1.0
  etcdctl-top removed by upstream
  resolves: #1416440

* Fri Jan 20 2017 d.marlin <dmarlin@redhat.com>
- Build for all archs (adding ppc64le and s390x)

* Tue Jan 10 2017 d.marlin <dmarlin@redhat.com> 
- Add aarch64 to ExclusiveArch list.

* Mon Jan 09 2017 d.marlin <dmarlin@redhat.com> 
- Correct 'link' warning for -X flag.

* Thu Dec 01 2016 jchaloup <jchaloup@redhat.com> - 3.0.15-1
- Update to 3.0.15
  Obsolete etcd3 < 3.0.15

* Fri Nov 18 2016 jchaloup <jchaloup@redhat.com> - 3.0.14-3
- Build with debug-info subpackage
- Until etcd3 obsoletes etcd it conflicts with it

* Tue Nov 15 2016 Avesh Agarwal <avagarwa@redhat.com> - 3.0.14-2
- Resolves: #1395359 etcd3 should not obsolete etcd

* Mon Nov 07 2016 jchaloup <jchaloup@redhat.com> - 3.0.14-1
- Update to v3.0.14
  related: #1386963

* Thu Oct 27 2016 jchaloup <jchaloup@redhat.com> - 3.0.13-1
- Update to v3.0.13
  related: #1386963

* Fri Oct 21 2016 jchaloup <jchaloup@redhat.com> - 3.0.12-3
- etcdctl: fix migrate in outputing client.Node to json
  resolves: #1386963

* Tue Oct 18 2016 jchaloup <jchaloup@redhat.com> - 3.0.12-2
- Replace etcd with etcd3 when upgrading
  resolves: #1384161

* Thu Oct 13 2016 jchaloup <jchaloup@redhat.com> - 3.0.12-1
- Update to v3.0.12

* Thu Oct 06 2016 jchaloup <jchaloup@redhat.com> - 3.0.10-1
- Update to v3.0.10

* Fri Sep 23 2016 jchaloup <jchaloup@redhat.com> - 3.0.3-2
- Extend etcd.conf with new flags
  resolves: #1378706

* Fri Jul 22 2016 jchaloup <jchaloup@redhat.com> - 3.0.2-1
- Update to v3.0.3
  related: #1347499

* Tue Jul 12 2016 jchaloup <jchaloup@redhat.com> - 3.0.2-1
- Update to v3.0.2
  related: #1347499

* Sun May 15 2016 jchaloup <jchaloup@redhat.com> - 3.0.0-0.1.beta0
- Build etcd3 v3.0.0-beta0 for AH 7.3
  resolves: #1347499
