#!/bin/bash

# The golang standard library bundles x/net/http2 code by mashing it into a single h2_bundle.go file
# that our own packages can't use, so we also have to vendor x/net/http2 ourselves.
# This file is in RHEL's golang RPM. 
#
# When RHEL provides us an updated golang RPM that fixes a CVE in x/net/http2, we're at risk of assuming
# a simple rebuild is enough to fix our impacted packages (that's how it works for stdlib CVEs).
# But it's not enough: our vendored x/net/http2 also needs to be updated. This script checks for changes
# in RHEL's bundled x/net/http2 bits and fails the build so that we know to perform that manual update.

LAST_KNOWN_HASH=$1
LAST_KNOWN_RPM=$2

CURRENT_HASH=$(sha256sum /usr/lib/golang/src/net/http/h2_bundle.go | awk '{print $1}')
CURRENT_RPM=$(rpm -qf /usr/lib/golang/src/net/http/h2_bundle.go)

echo "LAST_KNOWN_RPM: ${LAST_KNOWN_RPM}"
echo "LAST_KNOWN_HASH: ${LAST_KNOWN_HASH}"
echo "CURRENT_RPM: ${CURRENT_RPM}"
echo "CURRENT_HASH: ${CURRENT_HASH}"

if [[ $CURRENT_HASH == $LAST_KNOWN_HASH ]]; then
    echo "System h2_bundle.go provided by ${CURRENT_RPM} matches last known h2_bundle.go from ${LAST_KNOWN_RPM}."
    exit 0
fi

cat << EOF
System h2_bundle.go provided by ${CURRENT_RPM} DOES NOT match last known h2_bundle.go from ${LAST_KNOWN_RPM}.
This means it may include CVE fixes that this package's vendored x/net/http2 does not include.

In order to avoid undetected security regressions, please do the following:
    1. Identify changes to the system's h2_bundle.go by examining the RPM changelog below and/or directly comparing the source files.
    2. Ensure those changes are also applied to the x/net/http2 code vendored by this package (via patch files or vendor source tarball update).
    3. Only then update the LAST_KNOWN_HASH and LAST_KNOWN_RPM values in this script to allow this build to continue.
EOF

echo "Changelog for ${CURRENT_RPM}:"
rpm -q --changelog ${CURRENT_RPM}

exit 1
